@@ -61,23 +61,28 @@ is.circos.heatmap.cached = function() {
 # == param
 # -mat A matrix or a vector. The vector is transformed as a one-column matrix.
 # -split A categorical variable. It splits the matrix into a list of matrices.
-# -cluster whether to apply clustering on rows.
+# -cluster whether to apply clustering on rows. The value can also be a ``dendrogram``/``hclust`` object or other objects that
+#      can be converted to with `stats::as.dendrogram`.
 # -clustering.method Clustering method, pass to `stats::hclust`.
 # -distance.method Distance method, pass to `stats::dist`.
 # -dend.callback A callback function that is applied to the dendrogram in every sector.
+# -cell_width Relative widths of heatmap cells.
 # 
 # == seealso
 # https://jokergoo.github.io/2020/05/21/make-circular-heatmaps/
 #
 circos.heatmap.initialize = function(mat, split = NULL, cluster = TRUE, 
 	clustering.method = "complete", distance.method = "euclidean",
-	dend.callback = function(dend, m, si) reorder(dend, rowMeans(m))) {
+	dend.callback = function(dend, m, si) reorder(dend, rowMeans(m)),
+	cell_width = rep(1, nrow(mat))) {
 
-	if(is.null(split)) {
-		if(is.matrix(mat)) {
-			split = rep("mat", nrow(mat))
+	if(!is.matrix(mat)) {
+		if(is.vector(mat) && is.atomic(mat)) {
+			mat = matrix(mat, ncol = 1)
+		} else if(is.data.frame(mat)) {
+			mat = as.matrix(mat)
 		} else {
-			split = rep("mat", length(mat))
+			stop_wrap("The input should be a matrix or a vector.")
 		}
 	}
 
@@ -87,9 +92,72 @@ circos.heatmap.initialize = function(mat, split = NULL, cluster = TRUE,
 		}
 	}
 
-	mat_list = circos.heatmap.format.input(mat, split)
+	if(length(cell_width) == 1) {
+		cell_width = rep(cell_width, nrow(mat))
+	} else if(length(cell_width) < nrow(mat)) {
+		cell_width = rep(cell_width, times = nrow(mat))[1:nrow(mat)]
+	}
+
+	cell_width = cell_width/sum(cell_width)*nrow(mat)
+
+	if(identical(cluster, NA) || identical(cluster, NULL)) cluster = FALSE
+	if(!(identical(cluster, TRUE) || identical(cluster, FALSE))) {
+		cluster = as.dendrogram(cluster)
+		cluster_is_dendrogram = TRUE
+	} else {
+		if(identical(cluster, TRUE) && !is.null(split)) {
+			if(length(split) == 1) {
+				cluster = as.dendrogram(hclust(dist(mat, method = distance.method), method = clustering.method))
+				cluster_is_dendrogram = TRUE
+			} else {
+				cluster_is_dendrogram = FALSE
+			}
+		} else {
+			cluster_is_dendrogram = FALSE
+		}
+	}
+
+	if(cluster_is_dendrogram) {
+		if(is.null(split)) {
+			dend_list = list(cluster)
+			split2 = rep("group", nrow(mat))
+			names(dend_list) = split2[1]
+		} else {
+			if(length(split) > 1) {
+				stop_wrap("When `cluster` is specified as a clustering object, `split` can only be a single number.")
+			} else {
+				split2 = paste0("group", cutree(as.hclust(cluster), split))
+				dend_list = cut_into_k_dendrograms(cluster, split)
+				names(dend_list) = sapply(dend_list, function(d) {
+					split2[order.dendrogram(d)][1]
+				})
+
+				"order.dendrogram<-" = getFromNamespace("order.dendrogram<-", ns = "ComplexHeatmap")
+				dend_list = lapply(dend_list, function(d) {
+					order.dendrogram(d) = rank(order.dendrogram(d))
+					d
+				})
+			}
+		}
+	} else {
+		if(is.null(split)) {
+			split2 = rep("group", nrow(mat))
+		} else {
+			split2 = split
+		}
+	}
+
+	mat_list = circos.heatmap.format.input(mat, split2)
+	cell_width_list = split(cell_width, split2)
+	cell_width_list = cell_width_list[names(mat_list)]
 	n = length(mat_list)
 	subset_list = attr(mat_list, "subset_list")
+	if(exists("dend_list")) {
+		dend_list = dend_list[names(mat_list)]
+		for(nm in names(dend_list)) {
+			dend_list[[nm]] = dend.callback(dend_list[[nm]], mat_list[[nm]], nm)
+		}
+	}
 
 	cell.padding = c(0, 0, 0, 0)
 	track.margin = c(0.02, 0)
@@ -107,47 +175,92 @@ circos.heatmap.initialize = function(mat, split = NULL, cluster = TRUE,
 
 	circos.par(cell.padding = cell.padding, track.margin = track.margin, gap.degree = gap.degree)
 
-	# qqcat("initialize the circular plot with @{n} matrices.\n")
-	circos.initialize(names(mat_list), xlim = cbind(rep(0, n), sapply(mat_list, nrow)))
-
-	if(is.character(mat_list[[1]])) cluster = FALSE
+	circos.initialize(names(mat_list), xlim = cbind(rep(0, n), sapply(cell_width_list, sum)))
 
 	env = circos.par("__tempenv__")
-	if(cluster) {
-		# qqcat("perform clustering.\n")
-		dend_list = list()
-		for(nm in names(mat_list)) {
-			m = mat_list[[nm]]
-			dend = as.dendrogram(hclust(dist(m, method = distance.method), method = clustering.method))
-			dend_list[[nm]] = dend.callback(dend, m, nm)
-		}
 
+	if(cluster_is_dendrogram) {
 		for(se in get.all.sector.index()) {
-			add.sector.meta.data("row_dend", dend_list[[se]], sector.index = se)
-			add.sector.meta.data("dend", dend_list[[se]], sector.index = se)
-			add.sector.meta.data("row_order", order.dendrogram(dend_list[[se]]), sector.index = se)
-			add.sector.meta.data("order", order.dendrogram(dend_list[[se]]), sector.index = se)
+			dend = dend_list[[se]]
+			
+			od = order.dendrogram(dend)
+			cw = cell_width_list[[se]][od]
+			od2 = order(od)
+			add.sector.meta.data("cell_width", cw[od2], sector.index = se)
+			add.sector.meta.data("cell_middle", (cumsum(cw) - cw/2)[od2], sector.index = se)
+
+			dend = ComplexHeatmap::adjust_dend_by_x(dend, cumsum(cw) - cw/2)
+			add.sector.meta.data("row_dend", dend, sector.index = se)
+			add.sector.meta.data("dend", dend, sector.index = se)
+			add.sector.meta.data("row_order", od, sector.index = se)
+			add.sector.meta.data("order", od, sector.index = se)
 			if(!is.null(subset_list)) {
 				add.sector.meta.data("subset", subset_list[[se]], sector.index = se)
 			}
+
+			
 		}
 		env$circos.heatmap.cluster = TRUE
 	} else {
-		# qqcat("use the natural order.\n")
-		for(se in get.all.sector.index()) {
-			add.sector.meta.data("row_order", 1:nrow(mat_list[[se]]), sector.index = se)
-			add.sector.meta.data("order", 1:nrow(mat_list[[se]]), sector.index = se)
-			if(!is.null(subset_list)) {
-				add.sector.meta.data("subset", subset_list[[se]], sector.index = se)
+		if(is.character(mat_list[[1]])) cluster = FALSE
+		if(cluster) {
+			# qqcat("perform clustering.\n")
+			dend_list = list()
+			for(nm in names(mat_list)) {
+				m = mat_list[[nm]]
+				dend = as.dendrogram(hclust(dist(m, method = distance.method), method = clustering.method))
+				dend_list[[nm]] = dend.callback(dend, m, nm)
 			}
+
+			for(se in get.all.sector.index()) {
+				dend = dend_list[[se]]
+				
+				od = order.dendrogram(dend)
+				cw = cell_width_list[[se]][od]
+				od2 = order(od)
+				add.sector.meta.data("cell_width", cw[od2], sector.index = se)
+				add.sector.meta.data("cell_middle", (cumsum(cw) - cw/2)[od2], sector.index = se)
+
+				dend = ComplexHeatmap::adjust_dend_by_x(dend, cumsum(cw) - cw/2)
+				add.sector.meta.data("row_dend", dend, sector.index = se)
+				add.sector.meta.data("dend", dend, sector.index = se)
+				add.sector.meta.data("row_order", od, sector.index = se)
+				add.sector.meta.data("order", od, sector.index = se)
+				if(!is.null(subset_list)) {
+					add.sector.meta.data("subset", subset_list[[se]], sector.index = se)
+				}
+				
+			}
+			env$circos.heatmap.cluster = TRUE
+		} else {
+			# qqcat("use the natural order.\n")
+			for(se in get.all.sector.index()) {
+				add.sector.meta.data("row_order", 1:nrow(mat_list[[se]]), sector.index = se)
+				add.sector.meta.data("order", 1:nrow(mat_list[[se]]), sector.index = se)
+				if(!is.null(subset_list)) {
+					add.sector.meta.data("subset", subset_list[[se]], sector.index = se)
+				}
+				cw = cell_width_list[[se]]
+				add.sector.meta.data("cell_width", cw, sector.index = se)
+				add.sector.meta.data("cell_middle", cumsum(cw) - cw/2, sector.index = se)
+			}
+			env$circos.heatmap.cluster = FALSE
 		}
-		env$circos.heatmap.cluster = FALSE
 	}
-	env$circos.heatmap.split = split
+	env$circos.heatmap.split = split2
 	env$circos.heatmap.initialized = TRUE
 
 }
 
+cut_into_k_dendrograms = function(dend, k) {
+	if(!requireNamespace("ComplexHeatmap", quietly = TRUE)) {
+		stop_wrap("You need ComplexHeatmap package to process the pre-defined dendrogram. Please install it from Bioconductor.")
+	}
+
+	dl = getFromNamespace("cut_dendrogram", ns = "ComplexHeatmap")(dend, k)$lower
+	dl
+}
+
 # e.g. to check number of rows, split varaible, ...
 circos.heatmap.validate = function(mat_list) {
 
@@ -171,11 +284,15 @@ circos.heatmap.validate = function(mat_list) {
 # -col If the values in the matrices are continuous, the color should be a color mapping generated by 
 #      `colorRamp2`. If the values are characters, the color should be a named color vector.
 # -na.col Color for ``NA`` values.
+# -cell.border Border color of cells. A single scalar.
+# -cell.lty Line type of cell borders. A single scalar.
+# -cell.lwd Line width of cell borders. A single scalar.
 # -bg.border Color for background border.
 # -bg.lty Line type of the background border.
 # -bg.lwd Line width of the background border.
 # -ignore.white Whether to draw the white color?
-# -cluster whether to apply clustering on rows.
+# -cluster whether to apply clustering on rows. The value can also be a ``dendrogram``/``hclust`` object or other objects that
+#      can be converted to with `stats::as.dendrogram`.
 # -clustering.method Clustering method, pass to `stats::hclust`.
 # -distance.method Distance method, pass to `stats::dist`.
 # -dend.callback A callback function that is applied to the dendrogram in every sector.
@@ -186,6 +303,7 @@ circos.heatmap.validate = function(mat_list) {
 # -rownames.font Font of row names.
 # -rownames.col Color of row names.
 # -show.sector.labels Whether to show sector labels.
+# -cell_width Relative widths of heatmap cells.
 # -... Pass to `circos.track` which draws the heatmap track.
 # 
 # == seealso
@@ -209,25 +327,35 @@ circos.heatmap.validate = function(mat_list) {
 # circos.clear()
 # }
 circos.heatmap = function(mat, split = NULL, col, na.col = "grey", 
-	bg.border = NA, bg.lty = par("lty"), bg.lwd = par("lwd"), ignore.white = TRUE, 
+	cell.border = NA, cell.lty = 1, cell.lwd = 1,
+	bg.border = NA, bg.lty = par("lty"), bg.lwd = par("lwd"), 
+	ignore.white = is.na(cell.border), 
 	cluster = TRUE, clustering.method = "complete", distance.method = "euclidean",
 	dend.callback = function(dend, m, si) reorder(dend, rowMeans(m)),
 	dend.side = c("none", "outside", "inside"), dend.track.height = 0.1,
 	rownames.side = c("none", "outside", "inside"), rownames.cex = 0.5,
 	rownames.font = par("font"), rownames.col = "black", 
-	show.sector.labels = FALSE, ...) {
+	show.sector.labels = FALSE, cell_width = rep(1, nrow(mat)), ...) {
 
 	if(!is.matrix(mat)) {
-		if(is.vector(mat)) {
+		if(is.vector(mat) && is.atomic(mat)) {
 			mat = matrix(mat, ncol = 1)
+		} else if(is.data.frame(mat)) {
+			mat = as.matrix(mat)
+		} else {
+			stop_wrap("The input should be a matrix or a vector.")
 		}
 	}
 
+	env = circos.par("__tempenv__")
+
 	if(!is.circos.initialized()) {
 		cell.padding = c(0, 0, 0, 0)
 		track.margin = c(0.02, 0)
 		gap.degree = 2
 
+		env$circos.heatmap.split = NULL
+
 		if(!identical(circos.par("cell.padding"), c(0.02, 1, 0.02, 1))) {
 			cell.padding = circos.par("cell.padding")
 		}
@@ -242,11 +370,9 @@ circos.heatmap = function(mat, split = NULL, col, na.col = "grey",
 
 		circos.heatmap.initialize(mat, split = split, cluster = cluster, 
 			clustering.method = clustering.method, distance.method = distance.method,
-			dend.callback = dend.callback)
+			dend.callback = dend.callback, cell_width = cell_width)
 	}
 
-	env = circos.par("__tempenv__")
-	
 	split = env$circos.heatmap.split
 	mat_list = circos.heatmap.format.input(mat, split)
 
@@ -272,7 +398,7 @@ circos.heatmap = function(mat, split = NULL, col, na.col = "grey",
 		    panel.fun = function(x, y) {
 		        sector.numeric.index = get.cell.meta.data("sector.numeric.index")
 		        dend = dend_list[[sector.numeric.index]]
-		        circos.dendrogram(dend, max_height = max_height, facing = "inside")
+		        circos.dendrogram(dend, max_height = max_height, facing = "inside", use_x_attr = TRUE)
 		})
 	}
 
@@ -299,7 +425,8 @@ circos.heatmap = function(mat, split = NULL, col, na.col = "grey",
 				    nr = nrow(m)
 	    
 				    if(!is.null(rownames(m))) {
-				    	circos.text(1:nr - 0.5, rep(0, nr), rownames(m)[od], cex = rownames.cex[CELL_META$subset][od], 
+				    	circos.text(CELL_META$cell_middle[od], rep(0, nr), rownames(m)[od], 
+				    		cex = rownames.cex[CELL_META$subset][od], 
 				    		font = rownames.font[CELL_META$subset][od], col = rownames.col[CELL_META$subset][od],
 				    		facing = "clockwise", niceFacing = TRUE, adj = c(0, 0.5))
 				    }
@@ -316,7 +443,15 @@ circos.heatmap = function(mat, split = NULL, col, na.col = "grey",
 	}
 	
 	if(is.function(col)) {
-		col_fun = col
+		col_fun = function(x) {
+			attr = attributes(x)
+			l = is.na(x)
+			v = character(length(x))
+			v[!l] = col(x[!l])
+			v[l] = na.col
+			attributes(v) = attr
+			v 
+		}
 	} else {
 		if(is.null(names(col))) {
 			stop_wrap('`col` should be a named vector.')
@@ -348,25 +483,45 @@ circos.heatmap = function(mat, split = NULL, col, na.col = "grey",
 	    nr = nrow(m2)
 	    nc = ncol(m2)
 
+	    if(ignore.white) {
+		    if(any(grepl("#FFFFFF", col_mat)) && !is.na(cell.border)) {
+				stop_wrap("`ignore.white` must be set to `FALSE` when `cell.border` is set.")
+			}
+		}
+
 	    for(i in 1:nc) {
 	    	if(ignore.white) {
 	    		l = grepl("#FFFFFF", col_mat[, i])  # white colors
+
 	    		if(all(l)) {
 	    			# no nothing
 	    		} else {
 	    			# qqcat("@{sum(l)} white rectangles are not drawn.\n")
+	    			if(is.na(cell.border)) {
+		    			cell.border2 = col_mat[, i][!l]
+		    		} else {
+		    			cell.border2 = cell.border
+		    		}
 	    			circos.rect(
-	    				(1:nr - 1)[!l], 
+	    				(CELL_META$cell_middle - CELL_META$cell_width/2)[od][!l], 
 	    				(rep(nc - i, nr))[!l], 
-			            (1:nr)[!l], 
+			            (CELL_META$cell_middle + CELL_META$cell_width/2)[od][!l], 
 			            (rep(nc - i + 1, nr))[!l], 
-			            border = col_mat[, i][!l], 
+			            border = cell.border2, lty = cell.lty, lwd = cell.lwd,
 			            col = col_mat[, i][!l])
 	    		}
 	    	} else {
-		        circos.rect(1:nr - 1, rep(nc - i, nr), 
-		            1:nr, rep(nc - i + 1, nr), 
-		            border = col_mat[, i], col = col_mat[, i])
+	    		if(is.na(cell.border)) {
+	    			cell.border2 = col_mat[, i]
+	    		} else {
+	    			cell.border2 = cell.border
+	    		}
+		        circos.rect(
+		        	(CELL_META$cell_middle - CELL_META$cell_width/2)[od], 
+		        	rep(nc - i, nr), 
+		            (CELL_META$cell_middle + CELL_META$cell_width/2)[od], 
+		            rep(nc - i + 1, nr), 
+		            border = cell.border2, col = col_mat[, i], lty = cell.lty, lwd = cell.lwd)
 		       }
 	    }
 
@@ -388,7 +543,7 @@ circos.heatmap = function(mat, split = NULL, col, na.col = "grey",
 		    panel.fun = function(x, y) {
 		        sector.numeric.index = get.cell.meta.data("sector.numeric.index")
 		        dend = dend_list[[sector.numeric.index]]
-		        circos.dendrogram(dend, max_height = max_height, facing = "outside")
+		        circos.dendrogram(dend, max_height = max_height, facing = "outside", use_x_attr = TRUE)
 		})
 	}
 
@@ -402,7 +557,7 @@ circos.heatmap = function(mat, split = NULL, col, na.col = "grey",
 				    nr = nrow(m)
 	    
 				    if(!is.null(rownames(m))) {
-				    	circos.text(1:nr - 0.5, rep(1, nr), rownames(m)[od], cex = rownames.cex[CELL_META$subset][od], 
+				    	circos.text(CELL_META$cell_middle[od], rep(1, nr), rownames(m)[od], cex = rownames.cex[CELL_META$subset][od], 
 				    		font = rownames.font[CELL_META$subset][od], col = rownames.col[CELL_META$subset][od],
 				    		facing = "clockwise", niceFacing = TRUE, adj = c(1, 0.5))
 				    }
@@ -477,3 +632,33 @@ circos.heatmap.link = function(row_from, row_to, ...) {
 	circos.link(group1, x1 - 0.5, group2, x2 - 0.5, ...)
 
 }
+
+
+# == title
+# Get the x-position for heatmap rows
+#
+# == param
+# -row_ind A vector of row indicies.
+#
+# == value
+# A three-column data frame of the sector, the x-positions on the corresponding sectors, and the original row indicies.
+circos.heatmap.get.x = function(row_ind) {
+	env = circos.par("__tempenv__")
+	split = env$circos.heatmap.split
+
+	row_ind_lt = split(row_ind, split[row_ind])
+	row_ind_lt = row_ind_lt[sapply(row_ind_lt, length) > 0]
+	
+	x = NULL
+	for(i in row_ind_lt) {
+
+		subset = get.cell.meta.data("subset", sector.index = split[i[1]])
+		order = get.cell.meta.data("row_order", sector.index = split[i[1]])
+		
+		x = c(x, which((1:length(split))[subset][order] %in% i))
+	}
+	df = data.frame(sector = rep(names(row_ind_lt), times = sapply(row_ind_lt, length)), 
+		x = x - 0.5, row_ind = unlist(row_ind_lt))
+	rownames(df) = NULL
+	df
+}
