@@ -53,30 +53,51 @@ circos.link = function(
 	point1,
 	sector.index2,
 	point2,
-    rou = get_most_inside_radius(),
-    rou1 = rou,
-    rou2 = rou,
-    h = NULL,
-    h.ratio = 0.5,
-    w = 1,
-    h2 = h,
-    w2 = w,
-    inverse = FALSE,
-    col = "black",
-    lwd = par("lwd"),
-    lty = par("lty"),
-    border = col,
-    directional = 0,
-    arr.length = ifelse(arr.type == "big.arrow", 0.02, 0.4),
-    arr.width = arr.length/2,
-    arr.type = "triangle",
-    arr.lty = lty,
-    arr.lwd = lwd,
-    arr.col = col,
-    reduce_to_mid_line = FALSE) {
+	rou = get_most_inside_radius(),
+	rou1 = rou,
+	rou2 = rou,
+	h = NULL,
+	h.ratio = 0.5,
+	w = 1,
+	h2 = h,
+	w2 = w,
+	inverse = FALSE,
+	col = "black",
+	lwd = par("lwd"),
+	lty = par("lty"),
+	border = col,
+	directional = 0,
+	arr.length = ifelse(arr.type == "big.arrow", 0.02, 0.4),
+	arr.width = arr.length/2,
+	arr.type = "triangle",
+	arr.lty = lty,
+	arr.lwd = lwd,
+	arr.col = col,
+	reduce_to_mid_line = FALSE) {
 
-    sector.data1 = get.sector.data(sector.index1)
-    sector.data2 = get.sector.data(sector.index2)
+   sector.data1 = get.sector.data(sector.index1)
+   sector.data2 = get.sector.data(sector.index2)
+
+   if(is.data.frame(point1)) {
+   		point1 = unlist(point1)
+   }
+   if(is.data.frame(point2)) {
+   		point2 = unlist(point2)
+   }
+	
+   if(circos.par$ring) {
+   	if(length(point1) == 2) {
+   		if(point1[1] > point1[2]) {
+   			point1[1] = point1[1] - diff(get.sector.data()[c("min.value", "max.value")])
+   		}
+   	}
+
+   	if(length(point2) == 2) {
+   		if(point2[1] > point2[2]) {
+   			point2[1] = point2[1] - diff(get.sector.data()[c("min.value", "max.value")])
+   		}
+   	}
+   }
 
 	point1 = sort(point1)
 	point2 = sort(point2)
