@@ -107,6 +107,11 @@ reverse.circlize = function(
 
 	x = (sector.data["start.degree"] - theta) / abs(sector.data["end.degree"] - sector.data["start.degree"]) *
 	    (sector.data["max.value"] - sector.data["min.value"]) + sector.data["min.value"]
+
+    if(!circos.par("xaxis.clock.wise")) {
+        x = sector.data["max.value"] - x + sector.data["min.value"]
+    }
+
 	if(track.index > 0) {
         y = (cell.data$track.height - (cell.data$track.start - rou)) / cell.data$track.height * (cell.ylim[2] - cell.ylim[1]) + cell.ylim[1]
 	} else {
@@ -128,7 +133,7 @@ polar2Cartesian = function(d) {
 
 # expand breakpoints in two points to draw an arc
 # x and y are transformed and re-mapped points
-lines.expand = function(x, y, sector.index = get.current.sector.index(), track.index = get.current.track.index()) {
+lines_expand = function(x, y, sector.index = get.current.sector.index(), track.index = get.current.track.index()) {
     sector.data = get.sector.data(sector.index)
 	cell.data = get.cell.data(sector.index, track.index)
     nx = x[1]
@@ -253,7 +258,9 @@ as.degree = function(radian) {
 # -breaks A vector indicating numeric breaks
 # -colors A vector of colors which correspond to values in ``breaks``
 # -transparency A single value in ``[0, 1]``. 0 refers to no transparency and 1 refers to full transparency
-# -space color space in which colors are interpolated. Value should be one of "RGB", "HSV", "HLS", "LAB", "XYZ", "sRGB", "LUV", see `colorspace::color-class` for detail.
+# -space color space in which colors are interpolated. Value should be one of "RGB", "LAB", "XYZ", "sRGB", "LUV", see `colorspace::color-class` for detail.
+# -hcl_palette Name of the HCL palette. Value should be supported in `grDevices::hcl.pals`.
+# -reverse Whether should the colors in ``hcl_palette`` be reversed.
 #
 # == details
 # Colors are linearly interpolated according to break values and corresponding colors through CIE Lab color space (`colorspace::LAB`) by default. 
@@ -268,89 +275,112 @@ as.degree = function(radian) {
 # == example
 # col_fun = colorRamp2(c(-1, 0, 1), c("green", "white", "red"))
 # col_fun(c(-2, -1, -0.5, 0, 0.5, 1, 2))
-colorRamp2 = function(breaks, colors, transparency = 0, space = "LAB") {
+colorRamp2 = function(breaks, colors, transparency = 0, space = "LAB", 
+    hcl_palette = NULL, reverse = FALSE) {
 
-  if(length(breaks) != length(colors)) {
-    stop_wrap("Length of `breaks` should be equal to `colors`.\n")
-  }
-  
-  colors = colors[order(breaks)]
-  breaks = sort(breaks)
-
-  l = duplicated(breaks)
-  breaks = breaks[!l]
-  colors = colors[!l]
-
-  if(length(breaks) == 1) {
-    stop_wrap("You should have at least two distinct break values.")
-  } 
-
-
-  if(! space %in% c("RGB", "HSV", "HLS", "LAB", "XYZ", "sRGB", "LUV")) {
-    stop_wrap("`space` should be in 'RGB', 'HSV', 'HLS', 'LAB', 'XYZ', 'sRGB', 'LUV'")
-  }
-  
-  colors = t(col2rgb(colors)/255)
-
-  attr = list(breaks = breaks, colors = colors, transparency = transparency, space = space)
-
-  if(space == "LUV") {
-    i = which(apply(colors, 1, function(x) all(x == 0)))
-    colors[i, ] = 1e-5
-  }
-  
-  transparency = 1-ifelse(transparency > 1, 1, ifelse(transparency < 0, 0, transparency))[1]
-  transparency_str = sprintf("%X", round(transparency*255))
-  if(nchar(transparency_str) == 1) transparency_str = paste0("0", transparency_str)
-  
-  fun = function(x = NULL, return_rgb = FALSE, max_value = 1) {
-    if(is.null(x)) {
-      stop_wrap("Please specify `x`\n")
+    breaks_o = breaks
+    if(!is.null(hcl_palette)) {
+        breaks = sort(breaks)
+        if(length(breaks) == 2) {
+            breaks2 = seq(breaks[1], breaks[2], length = 1000)
+            colors = hcl.colors(1000, palette = hcl_palette, rev = reverse)
+        } else {
+            nb = length(breaks)
+            nl = ceiling(1000/nb)
+            breaks2 = seq(breaks[1], breaks[2], length = nl)
+            for(i in 2:(nb-1)) {
+                breaks2  = c(breaks2, seq(breaks[i], breaks[i+1], length = nl)[-1])
+            }
+            colors = hcl.colors(length(breaks2), palette = hcl_palette, rev = reverse)
+        }
+        breaks_o = breaks
+        breaks = breaks2
     }
 
-    att = attributes(x)
-    if(is.data.frame(x)) x = as.matrix(x)
-
-    l_na = is.na(x)
-    if(all(l_na)) {
-      return(rep(NA, length(l_na)))
+    if(length(breaks) != length(colors)) {
+        stop_wrap("Length of `breaks` should be equal to `colors`.\n")
     }
 
-    x2 = x[!l_na]
+    od = order(breaks)
+    breaks = breaks[od]
+
+    colors = colors[od]
     
-    x2 = ifelse(x2 < breaks[1], breaks[1],
-               ifelse(x2 > breaks[length(breaks)], breaks[length(breaks)],
-                      x2
-               ))
-    ibin = .bincode(x2, breaks, right = TRUE, include.lowest = TRUE)
-    res_col = character(length(x2))
-    for(i in unique(ibin)) {
-      l = ibin == i
-      res_col[l] = .get_color(x2[l], breaks[i], breaks[i+1], colors[i, ], colors[i+1, ], space = space)
-    }
-    res_col = paste(res_col, transparency_str[1], sep = "")
-    
-    if(return_rgb) {
-      res_col = t(col2rgb(as.vector(res_col), alpha = TRUE)/255)
-      return(res_col)
-    } else {
-      res_col2 = character(length(x))
-      res_col2[l_na] = NA
-      res_col2[!l_na] = res_col
+    l = duplicated(breaks)
+    breaks = breaks[!l]
+    colors = colors[!l]
 
-      attributes(res_col2) = att
-      return(res_col2)
+    if(length(breaks) == 1) {
+        stop_wrap("You should have at least two distinct break values.")
+    } 
+
+
+    if(! space %in% c("RGB", "LAB", "XYZ", "sRGB", "LUV", "HLS")) {
+        stop_wrap("`space` should be in 'RGB', 'LAB', 'XYZ', 'sRGB', 'LUV'")
     }
-  }
-  
-  attributes(fun) = attr
-  return(fun)
+
+    colors = t(col2rgb(colors)/255)
+
+    
+    if(space == "LUV") {
+        i = which(apply(colors, 1, function(x) all(x == 0)))
+        colors[i, ] = 1e-5
+    }
+
+    transparency = 1-ifelse(transparency > 1, 1, ifelse(transparency < 0, 0, transparency))[1]
+    transparency_str = sprintf("%X", round(transparency*255))
+    if(nchar(transparency_str) == 1) transparency_str = paste0("0", transparency_str)
+
+    fun = function(x = NULL, return_rgb = FALSE, max_value = 1) {
+        if(is.null(x)) {
+            stop_wrap("Please specify `x`\n")
+        }
+
+        att = attributes(x)
+        if(is.data.frame(x)) x = as.matrix(x)
+
+        l_na = is.na(x)
+        if(all(l_na)) {
+            return(rep(NA, length(l_na)))
+        }
+
+        x2 = x[!l_na]
+
+        x2 = ifelse(x2 < breaks[1], breaks[1],
+                   ifelse(x2 > breaks[length(breaks)], breaks[length(breaks)],
+                          x2
+                   ))
+        ibin = .bincode(x2, breaks, right = TRUE, include.lowest = TRUE)
+        res_col = character(length(x2))
+        for(i in unique(ibin)) {
+            l = ibin == i
+            res_col[l] = .get_color(x2[l], breaks[i], breaks[i+1], colors[i, ], colors[i+1, ], space = space)
+        }
+        res_col = paste(res_col, transparency_str[1], sep = "")
+
+        if(return_rgb) {
+            res_col = t(col2rgb(as.vector(res_col), alpha = TRUE)/255)
+            return(res_col)
+        } else {
+            res_col2 = character(length(x))
+            res_col2[l_na] = NA
+            res_col2[!l_na] = res_col
+
+            attributes(res_col2) = att
+            return(res_col2)
+        }
+    }
+
+    attr = list(breaks = breaks_o, colors = fun(breaks_o), transparency = 1-transparency, space = space)
+
+    attributes(fun) = attr
+    return(fun)
 }
 
 .restrict_in = function(x, lower, upper) {
-  x[x > upper] = upper
-  x[x < lower] = lower
-  x
+    x[x > upper] = upper
+    x[x < lower] = lower
+    x
 }
 
 # x: vector
@@ -360,21 +390,21 @@ colorRamp2 = function(breaks, colors, transparency = 0, space = "LAB") {
 # rgb2 vector with 3 elements
 .get_color = function(x, break1, break2, col1, col2, space) {
 
-  col1 = coords(as(sRGB(col1[1], col1[2], col1[3]), space))
-  col2 = coords(as(sRGB(col2[1], col2[2], col2[3]), space))
+    col1 = coords(as(sRGB(col1[1], col1[2], col1[3]), space))
+    col2 = coords(as(sRGB(col2[1], col2[2], col2[3]), space))
 
-  res_col = matrix(ncol = 3, nrow = length(x))
-  for(j in 1:3) {
+    res_col = matrix(ncol = 3, nrow = length(x))
+    for(j in 1:3) {
     xx = (x - break2)*(col2[j] - col1[j]) / (break2 - break1) + col2[j]
     res_col[, j] = xx
-  }
-  
-  res_col = get(space)(res_col)
-  res_col = coords(as(res_col, "sRGB"))
-  res_col[, 1] = .restrict_in(res_col[,1], 0, 1)
-  res_col[, 2] = .restrict_in(res_col[,2], 0, 1)
-  res_col[, 3] = .restrict_in(res_col[,3], 0, 1)
-  hex(sRGB(res_col))
+    }
+
+    res_col = get(space)(res_col)
+    res_col = coords(as(res_col, "sRGB"))
+    res_col[, 1] = .restrict_in(res_col[,1], 0, 1)
+    res_col[, 2] = .restrict_in(res_col[,2], 0, 1)
+    res_col[, 3] = .restrict_in(res_col[,3], 0, 1)
+    hex(sRGB(res_col))
 }
 
 # will be considered in the future
@@ -540,7 +570,7 @@ adjacencyMatrix2List = function(mat, keep.zero = FALSE) {
 # Zuguang Gu <z.gu@dkfz.de>
 #
 # == example
-# x = seq(0, 1, length = 11)
+# x = seq(0, 1, length.out = 11)
 # col_fun = colorRamp2(c(0, 0.5, 1), c("blue", "white", "red"))
 # col = col_fun(x)
 # col2value(col, col_fun = col_fun)
@@ -575,6 +605,10 @@ col2value = function(r, g, b, col_fun) {
 
     n = length(r)
 
+    if(inherits(colors, "character")) {
+        colors = t(col2rgb(colors))/255
+    }
+
     ## convert all colors to the specified space
     m = coords(as(sRGB(r, g, b), space))
     breaks_m = coords(as(sRGB(colors), space))
@@ -1155,7 +1189,7 @@ validate_data_frame = function(x) {
 
 
 roundrect_pos = function(xleft, ybottom, xright, ytop, radius = 0.1, 
-  asp = NULL) {
+  asp = NULL, plot = FALSE) {
 
   if(is.null(asp)) {
     pin = par("pin")
@@ -1193,7 +1227,7 @@ roundrect_pos = function(xleft, ybottom, xright, ytop, radius = 0.1,
   y = NULL
 
   # left top 180 -> 90
-  df = polar2Cartesian(cbind(seq(180, 90, length = 20), rep(r,20)))
+  df = polar2Cartesian(cbind(seq(180, 90, length.out = 20), rep(r,20)))
   x = c(x, df[, 1] + xleft + r)
   y = c(y, df[, 2] + ytop - r)
 
@@ -1201,7 +1235,7 @@ roundrect_pos = function(xleft, ybottom, xright, ytop, radius = 0.1,
   y = c(y, ytop, ytop)
 
   # right top 90 -> 0
-  df = polar2Cartesian(cbind(seq(90, 0, length = 20), rep(r,20)))
+  df = polar2Cartesian(cbind(seq(90, 0, length.out = 20), rep(r,20)))
   x = c(x, df[, 1] + xright - r)
   y = c(y, df[, 2] + ytop - r)
 
@@ -1209,7 +1243,7 @@ roundrect_pos = function(xleft, ybottom, xright, ytop, radius = 0.1,
   y = c(y, ytop - r, ybottom + r)
 
   # bottom right 0 - (-90)
-  df = polar2Cartesian(cbind(seq(0, -90, length = 20), rep(r,20)))
+  df = polar2Cartesian(cbind(seq(0, -90, length.out = 20), rep(r,20)))
   x = c(x, df[, 1] + xright - r)
   y = c(y, df[, 2] + ybottom + r)
 
@@ -1217,7 +1251,7 @@ roundrect_pos = function(xleft, ybottom, xright, ytop, radius = 0.1,
   y = c(y, ybottom, ybottom)
 
   # bottom left 270 - 180
-  df = polar2Cartesian(cbind(seq(270, 180, length = 20), rep(r,20)))
+  df = polar2Cartesian(cbind(seq(270, 180, length.out = 20), rep(r,20)))
   x = c(x, df[, 1] + xleft + r)
   y = c(y, df[, 2] + ybottom + r)
 
@@ -1226,6 +1260,8 @@ roundrect_pos = function(xleft, ybottom, xright, ytop, radius = 0.1,
 
   y = covnert_y_2(y, asp)
 
+  if(plot) plot(x, y, type = "l")
+
   return(list(x = x, y = y))
 }
 
@@ -1238,3 +1274,197 @@ roundrect_pos = function(xleft, ybottom, xright, ytop, radius = 0.1,
 # foo = roundrect_pos(-4, -2, -1, -1)
 # lines(foo$x, foo$y)
 
+
+
+stack_text = function(x, text, margin = 0.2, 
+    strwidth = strwidth, strheight = strheight, ...) {
+
+    tw = sapply(text, strwidth, ...)
+    th = sapply(text, strheight, ...)
+
+    e = min(c(tw, th))*margin
+
+    tw = tw + e
+
+    x1 = x - tw/2
+    x2 = x + tw/2
+    ind = seq_along(x)
+
+    od = order(x1)
+    tw = tw[od]
+    x1 = x1[od]
+    x2 = x2[od]
+    ind = ind[od]
+
+    lt = list()
+
+    i_lt = 1
+    while(length(x1)) {
+        v = ind[1]
+        x_right = x2[1]
+
+        ind_used = 1
+        for(i in seq_along(x1)[-1]) {
+            if(x1[i] >= x_right) {
+                v = c(v, ind[i])
+                x_right = x2[i]
+                ind_used = c(ind_used, i)
+            }
+        }
+        x1 = x1[-ind_used]
+        x2 = x2[-ind_used]
+        ind = ind[-ind_used]
+
+        lt[[i_lt]] = v
+
+        i_lt = i_lt + 1
+
+    }
+
+    lt
+}
+
+
+circos.strwidth = function(text, ..., 
+    sector.index = get.current.sector.index(),
+    track.index = get.current.track.index(),
+    h = get.cell.meta.data("ylim", sector.index = sector.index, track.index = track.index)[1]) {
+
+    chars = strsplit(text, "")
+        
+    nlabel = length(text)
+    strw = lapply(chars, strwidth, ...) # width of every character in text
+    
+    strw = sapply(strw, function(w) {
+        w2 = convert_unit_in_data_coordinate(w, unit = "canvas", direction = "x",
+            sector.index = sector.index, track.index = track.index, h = h)
+        sum(w2)
+    })
+
+    strw
+}
+
+circos.strheight = function(text, ..., 
+    sector.index = get.current.sector.index(),
+    track.index = get.current.track.index()) {
+
+    strh = lapply(text, strheight, ...)
+    
+    strh = sapply(strh, function(w) {
+        w2 = convert_unit_in_data_coordinate(w, unit = "canvas", direction = "y",
+            sector.index = sector.index, track.index = track.index)
+        max(w2)
+    })
+
+    strh
+}
+
+circos.strAscent = function(text, cex = par("cex"), font = par("font"), family = par("family"),
+    sector.index = get.current.sector.index(),
+    track.index = get.current.track.index()) {
+
+    sapply(text, function(t) {
+        gb = textGrob(t, gp = gpar(cex = cex, fontface = font, fontfamily = family))
+        h = grobAscent(gb)
+        h = convertHeight(h, "mm", valueOnly = TRUE)
+        mm_y(h, sector.index = sector.index, track.index = track.index)
+    })
+}
+
+circos.strDescent = function(text, cex = par("cex"), font = par("font"), family = par("family"),
+    sector.index = get.current.sector.index(),
+    track.index = get.current.track.index()) {
+
+    sapply(text, function(t) {
+        gb = textGrob(t, gp = gpar(cex = cex, fontface = font, fontfamily = family))
+        h = grobDescent(gb)
+        h = convertHeight(h, "mm", valueOnly = TRUE)
+        mm_y(h, sector.index = sector.index, track.index = track.index)
+    })
+}
+
+## code copied from circos.genomicLabels()
+refer_to_one_sector = function(sectors, x) {
+
+    bed = data.frame(sectors = sectors, x = x, order = seq_along(x))
+    od = order(bed[[1]], bed[[2]])
+    bed = bed[od, , drop = FALSE]
+
+    if(!circos.par$xaxis.clock.wise) {
+        all_chr_vec = as.character(bed[, 1])
+        bed[, 2] = .CIRCOS.ENV$.SECTOR.DATA[all_chr_vec, "max.value"] - bed[, 2] + .CIRCOS.ENV$.SECTOR.DATA[all_chr_vec, "min.value"]
+        bed[, 3] = .CIRCOS.ENV$.SECTOR.DATA[all_chr_vec, "max.value"] - bed[, 3] + .CIRCOS.ENV$.SECTOR.DATA[all_chr_vec, "min.value"]
+        bed[, 2:3] = bed[, 3:2]
+        circos.par(xaxis.clock.wise = TRUE)
+        on.exit(circos.par(xaxis.clock.wise = FALSE))
+    }
+
+    chr = get.all.sector.index()[1]
+    sector_data = get.sector.data(chr)
+    
+    all_chr = unique(bed[, 1])
+    rho = NULL
+    for(cr in all_chr) {
+        sub_bed = bed[bed[, 1] == cr, ]
+        rho = c(rho, circlize(sub_bed[, 2], y = rep(1, nrow(sub_bed)), sector.index = cr)[, 1])
+    }
+    if(length(rho) == 0) {
+        anchor = ((sector_data["start.degree"] + sector_data["end.degree"])/2 + 180) %% 360
+    } else if(length(rho) == 1) {
+        anchor = (rho + 180) %% 360
+    } else {
+        rho = sort(rho)
+        rho = c(rho, rho[1] + 360)
+        i = which.max(diff(rho))
+        anchor = ((rho[i] + rho[i+1])/2) %% 360
+    }
+
+    # map all other chromosomes to the first chromosome
+    chr_width = sector_data["start.degree"] - sector_data["end.degree"]
+    # extend = (360 - chr_width)/chr_width
+    # extend = c(0, extend)
+    extend = numeric(2)
+    s1 = sector_data["start.degree"] %% 360
+    s2 = sector_data["end.degree"] %% 360
+    # if the anchor in inside the first sector
+    if(s1 < s2) { # the first sector go across theta = 0
+        s1 = s1 + 360
+    } 
+
+    if(anchor >= s2 && anchor <= s1) { # anchor inside sector
+        if(s1 - s2 > 180) {
+            extend[1] = (abs(s1 - anchor) %% 360)/chr_width
+            extend[2] = -(abs(s2 - anchor) %% 360)/chr_width
+        } else {
+            extend = (360 - chr_width)/chr_width
+            extend = c(0, extend)
+        }
+    } else {
+        extend[1] = ((anchor - s1) %% 360)/chr_width  # goes reverse clockwise
+        extend[2] = ((s2 - anchor) %% 360)/chr_width  # goes clockwise
+    }
+    extend = abs(extend)
+
+    if(0) segments(0, 0, cos(anchor/180*pi), sin(anchor/180*pi))
+
+    # start.degree is always larger than end.degree
+    new_chr_range = c(sector_data["start.degree"] + chr_width*extend[1], sector_data["end.degree"] - chr_width*extend[2])
+
+    all_chr = unique(bed[, 1])
+    bed2 = NULL
+    for(cr in all_chr) {
+        sub_bed = bed[bed[, 1] == cr, ]
+        if(cr != chr) {
+            dfx1 = circlize(sub_bed[, 2], y = rep(1, nrow(sub_bed)), sector.index = cr)
+            
+            l = dfx1[, 1] > new_chr_range[1] | dfx1[, 1] < new_chr_range[2]
+            if(any(l)) dfx1[l, 1] = (dfx1[l, 1] - new_chr_range[2]) %% 360 + new_chr_range[2]
+            x1 = reverse.circlize(dfx1, sector.index = chr)[, 1]
+            
+            sub_bed[, 2] = data.frame(start = x1)
+        }
+        bed2 = rbind(bed2, sub_bed)
+    }
+    bed2[, 1] = chr
+    bed2
+}
